/*

droneAI.js

Priority-based AI for Drones (NPC-drone, for player use other AI from the Armoury or Drones OXPs)

Oolite
Copyright © 2004-2013 Giles C Williams and contributors

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
MA 02110-1301, USA.

*/

"use strict";

this.name = "ASP Drone AI";

this.aiStarted = function() {
	var ai = new worldScripts["oolite-libPriorityAI"].PriorityAIController(this.ship);

	// add our custom AI so we can use all the AI properties via "this"
	ai.conditionMasterCloser = this.$conditionMasterCloser;
	ai.conditionScannerContainsReadyMaster = this.$conditionScannerContainsReadyMaster;
	ai.conditionIsActiveDrone = this.$conditionIsActiveDrone;
	ai.behaviourFastFollowGroupLeader = this.$behaviourFastFollowGroupLeader;

	// preconfiguration: setting a bounty, group, roles and etc.
	this.$configurationSetMaster();

	if (this.ship.scanClass == "CLASS_POLICE")
	{
		ai.setParameter("oolite_friendlyRoles",["oolite-trader","oolite-bounty-hunter","oolite-scavenger","oolite-shuttle"]);
	}
	ai.setParameter("oolite_flag_scanIgnoresUnpowered",true);

	ai.setPriorities([
		/* Check a master */
		{
			label: "Has Mothership?",
			condition: ai.conditionHasMothership,
			truebranch: [
				/* Survival */
				{
					label: "Pre-set Remote Control; Cascade Detected?",
					preconfiguration: ai.configurationSetRemoteControl,
					condition: ai.conditionCascadeDetected,
					behaviour: ai.behaviourAvoidCascadeExplosion,
					reconsider: 5
				},
				/* Fight: drone */
				// {
				// 	label: "Master in Combat: Destroy Target?",
				// 	condition: ai.conditionMothershipInCombat,
				// 	configuration: ai.configurationAcquireOffensiveEscortTarget,
				// 	behaviour: ai.behaviourDestroyCurrentTarget,
				// 	reconsider: 5
				// },
				/* Fight: combat drone */
				{
					label: "Master is Attacking Hostile?",
					condition: ai.conditionMothershipIsAttackingHostileTarget,
					configuration: ai.configurationAcquireCombatTarget,
					behaviour: ai.behaviourRepelCurrentTarget,
					reconsider: 5
				},
				{
					label: "Combat?",
					condition: ai.conditionInCombat,
					configuration: ai.configurationAcquireCombatTarget,
					behaviour: ai.behaviourRepelCurrentTarget,
					reconsider: 5
				},
				/* Battle is over; return to master */
				{
					label: "Master is not Closer: Follow to Master?",
					notcondition: ai.conditionMasterCloser,
					configuration: ai.configurationSetDestinationToGroupLeader,
					behaviour: ai.behaviourFastFollowGroupLeader,
					reconsider: 5
				},
				{
					label: "Escorting",
					behaviour: ai.behaviourEscortMothership,
					reconsider: 20
				}
			],
			falsebranch: [
				{
					label: "Find a Ready Master: Offer To Escort?",
				 	preconfiguration: ai.configurationCheckScanner,
				 	condition: ai.conditionScannerContainsReadyMaster,
				 	behaviour: ai.behaviourOfferToEscort,
				 	reconsider: 5
				},
				{
					label: "Become Inactive?",
					condition: ai.conditionIsActiveDrone,
					behaviour: ai.behaviourBecomeInactiveThargon,
					reconsider: 10
				},
				{
					label: "Tumble",
					behaviour: ai.behaviourTumble
				}
			]
		}
	]);
}

this.$conditionIsActiveDrone = function()
{
	return (this.ship.scanClass == "CLASS_NEUTRAL" || this.ship.scanClass == "CLASS_BUOY") && this.ship.hasRole("drone");
}

this.$conditionMasterCloser = function()
{
	return this.distance(this.ship.master) < this.scannerRange * 0.5; // for 'CT_thargon' the scanner range was set 25km
}

this.$conditionScannerContainsReadyMaster = function()
{
	return this.checkScannerWithPredicate(function(s) { 
		return s == this.ship.master && (!s.escortGroup || s.escortGroup.count <= 16); //return s.hasRole("thargoid-mothership") && (!s.escortGroup || s.escortGroup.count <= 16);
	});
}

// Setting in the initial startup AI
this.$configurationSetMaster = function()
{
	var master = this.ship.master;
	if (master && master.isValid && !master.isDerelict)
	{	
		this.ship.accuracy = master.accuracy;

		var bounty = 0; // for adding to escort group
		if (master.bounty > 0) {
			bounty = 30 + ~~(Math.random() * 30);
		}
		this.ship.bounty = bounty;

		var grp = master.group;
		if (grp == null)
		{
			//log("No master group, creating a ship group");
			grp = new ShipGroup(master.primaryRole + " group",master);
			master.group = grp;
		}

		// need for adding to escort group
		if (master.maxEscorts < 16)
		{
			master.maxEscorts++; // master.escortGroup.addShip(this.ship);
		}
		this.ship.primaryRole = "escort";
		//log("Master: " + master.name + ", maxEscorts: " + master.maxEscorts);

		if (master.target && master.target.isShip) this.ship.target = master.target;
		//log("Setting drone, group: " + this.ship.group + ", bounty: " + this.ship.bounty + ", role: " + this.ship.primaryRole + ", class: " + this.ship.scanClass + ", target: " + this.ship.target);				
	}
}

/* Follow the group leader in a less organised way than escorting them */
// Almost like behaviourFollowGroupLeader, but will added fast desired speed depend on from distance to leader,
// This function good use for fast escorts moving from far distance.
this.$behaviourFastFollowGroupLeader = function()
{	
	if (!this.ship.group || !this.ship.group.leader)
	{
		var handlers = {};
		this.responsesAddStandard(handlers);
		this.applyHandlers(handlers);
		this.ship.performIdle();
	}
	else
	{
		var gl = this.ship.group.leader;
		this.ship.destination = gl.position.add(gl.vectorForward.multiply(gl.speed*10));
		this.ship.desiredRange = 500+Math.random()*1000;
		var dist = this.distance(gl);
		var inj = ~~(dist * 0.001)/10 - 2.5;
		if (inj > 0)
		{
			this.ship.desiredSpeed = this.ship.maxSpeed*(1+inj); // little, use injectors if possible
		}
		else
		{
			this.ship.desiredSpeed = Math.min(this.ship.maxSpeed,gl.speed*1.5);
		}
		this.behaviourApproachDestination();
	}
} 

